/**
 * @license
 * Copyright Akveo. All Rights Reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 */
import { Component, OnInit } from '@angular/core';
import { ActivationEnd, NavigationEnd, NavigationStart, Router } from '@angular/router';
import { NbMenuItem } from '@nebular/theme';
import { SidebarNavService, SidebarNav } from './core/services/sidebar-nav.service';
import { BaseComponent } from './shared/components/base/base.component';
import { combineLatest } from 'rxjs';
import { takeUntil, filter } from 'rxjs/operators';
import { AuthService } from './core/services/auth.service';
import { AppStateService } from './core/services/app.state.service';
import { LocationStrategy } from '@angular/common';
import { LocationState } from './core/services/location-state';
import { BackNavigationService } from './core/services/back-navigation.service';

@Component({
  selector: 'ngx-app',
  styleUrls: ['./app.component.scss'],
  template: `
  <div id="root">
    <div class="content">
      <ngx-one-column-layout [sideNav]="sideNav" *ngIf="init">
        <router-outlet appRouteReuseLifecycle></router-outlet>
      </ngx-one-column-layout>
    </div>
    <div class="overlay">
    </div>
  </div>
  `,
})
export class AppComponent extends BaseComponent {
  sideNav: SidebarNav;

  init = false;

  constructor(
    private authService: AuthService,
    private appStateService: AppStateService,
    sidebarNavService: SidebarNavService,
    location: LocationStrategy,
    locationState: LocationState,
    private router: Router) {
    super();

    location.onPopState(() => {
      locationState.justNavigatedBack = true
    });

    this.router.events
      .pipe(
        takeUntil(this.unsubscribe),
        filter(e => e instanceof ActivationEnd),
      )
      .subscribe(e => {
        locationState.justNavigatedBack = false;
      });

    sidebarNavService.nav$
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(menu => {
        this.sideNav = menu;
      });

    router.events
      .pipe(takeUntil(this.unsubscribe))
      .subscribe((val) => {
        if (val instanceof NavigationStart) {
          if (!this.init) {
            this.initApp(val.url);
          }
        }
      });

    appStateService.init$
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(val => {
        this.init = val;
      });
  }

  initApp(url: string): void {
    combineLatest(this.authService.loadPrincipal(), this.appStateService.loadTerm(), this.appStateService.loadConfig())
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(([principal, term, config]) => {
        this.appStateService.setInit(true);
        if (principal != null && this.appStateService.currentUrl == '/') {
          this.router.navigate(["/dashboard"]);
        }
      });
  }
}
