import { User } from "app/core/models/user";
import { DegreeProgram } from "app/features/modules/models/degree-program";
import { ProgramModule } from "app/features/modules/models/program-module";
import { Term } from "app/features/modules/models/term";
import { TaskPool } from "app/features/tasks/models/task-pool";
import { CourseUnit } from "./course-unit";
import { CourseVisibility } from "./course-visibility";
import { SubmitPolicy } from "./submit-policy";

export class Course {
    id: number;
    name: string;
    shortName: string;
    description: string;
    motd: string;
    visibility: CourseVisibility;
    submitPolicy: SubmitPolicy;
    lecturers: User[];
    admins: User[];
    tutors: User[];
    modules: ProgramModule[];
    term: Term;
    showEmails: boolean;

    constructor(course: any) {
        this.id = course.id;
        this.name = course.name;
        this.description = course.description;
        this.shortName = course.shortName;
        this.motd = course.motd;
        this.showEmails = course.showEmails;
        this.visibility = course.visibility;
        this.submitPolicy = course.submitPolicy;
        this.modules = course.modules ? course.modules.map(mod => new ProgramModule(mod)) : [];
        this.lecturers = course.lecturers ? course.lecturers.map(lecturer => new User(lecturer)) : [];
        this.admins = course.admins ? course.admins.map(admin => new User(admin)) : [];
        this.tutors = course.tutors ? course.tutors.map(tutor => new User(tutor)) : [];
        this.term = course.term ? new Term(course.term) : null;
    }

    get adminNames(): string {
        return this.admins.map(u => u.name).join(', ');
    }

    moduleForProgram(program: DegreeProgram) {
        return this.modules.find(m => m.program.id == program.id);
    }

    get public() {
        return this.visibility == CourseVisibility.Public;
    }

    get internal() {
        return this.visibility == CourseVisibility.Internal;
    }

    get participantsOnly() {
        return this.visibility == CourseVisibility.Participants;
    }

    get private() {
        return this.visibility == CourseVisibility.Private;
    }
}