import { Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { FormArray, FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { User } from 'app/core/models/user';
import { AppStateService } from 'app/core/services/app.state.service';
import { AuthService } from 'app/core/services/auth.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { BackendError } from 'app/shared/models/backend-error';
import { NotificationService } from 'app/shared/services/notification.service';
import { environment } from 'environments/environment';
import { combineLatest, forkJoin, of } from 'rxjs';
import { takeUntil, flatMap, filter, catchError } from 'rxjs/operators';
import { Course } from '../../models/course';
import { CourseService } from '../../services/course.service';

@Component({
  selector: 'ngx-permissions',
  templateUrl: './permissions.component.html',
  styleUrls: ['./permissions.component.scss']
})
export class PermissionsComponent extends BaseComponent implements OnInit {
  course: Course;
  loading: boolean = true;
  principal: SubatoPrincipal;
  permissionManualUrl = environment.permissionManualUrl;
  
  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diesen Kurs konnten wir leider nicht finden.'
    }
  };
  
  constructor(
    private courseService: CourseService,
    private appStateService: AppStateService,
    private authService: AuthService,
    private route: ActivatedRoute
  ) {
    super();
  }

  ngOnInit(): void {
    combineLatest(this.route.paramMap, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(([params, principal]) => {
          this.principal = principal;
          let courseId = Number(params.get("course_id"));

          return this.courseService.getById(courseId);
        }),
      )
      .subscribe(course => {
        this.course = course;
        this.appStateService.setCourse(course);

        this.loading = false;
      }, (error) => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }
}
