import { HttpClient } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { HttpCacheManager } from "@ngneat/cashew";
import { BackendError } from "app/shared/models/backend-error";
import { BaseService } from "app/shared/services/base.service";
import { environment } from "environments/environment";
import { Observable } from "rxjs";
import { catchError, map } from "rxjs/operators";
import { UpdateEventCancellationDto } from "../dto/update-event-cancellation.dto";
import { CourseUnitEvent } from "../models/course-unit-event";

@Injectable({
    providedIn: 'root'
})
export class CourseUnitEventService extends BaseService {
    constructor(private http: HttpClient, private cacheManager: HttpCacheManager) {
        super();
    }

    updateCancellation(id: number, dto: UpdateEventCancellationDto): Observable<CourseUnitEvent> {
        return this.http.put<CourseUnitEvent>(`${environment.backendUrl}/course_unit_event/${id}/cancellation`, dto).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(event => {
                this.cacheManager.clear();
                return new CourseUnitEvent(event)
            })
        );
    }
}
