import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { Router } from '@angular/router';
import { TaskInstance } from '../../models/task-instance';
import { ExerciseProgress } from '../../models/exercise-progress';
import { Subject } from 'rxjs';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { takeUntil, debounceTime } from 'rxjs/operators';
import { ExerciseProgressResult } from '../../models/exercise-progress-result';

@Component({
  selector: 'ngx-exercise-progress',
  templateUrl: './exercise-progress.component.html',
  styleUrls: ['./exercise-progress.component.scss']
})
export class ExerciseProgressComponent extends BaseComponent implements OnInit {
  @Input() exerciseProgressResult: ExerciseProgressResult;
  @Input() showFilter: boolean;
  @Input() hideInvisible: boolean;

  @Input() set exerciseProgresses(val: ExerciseProgress[]) {
    this._exerciseProgresses = val;
    this.applyFilter();
  }

  get exerciseProgresses(): ExerciseProgress[] {
    return this._exerciseProgresses;
  }

  @Output() hideInvisibleChanged: EventEmitter<boolean> = new EventEmitter<boolean>();

  _exerciseProgresses: ExerciseProgress[] = [];
  filteredExerciseProgresses: ExerciseProgress[] = [];
  query: string;
  private queryChanged: Subject<string> = new Subject<string>();

  constructor(private router: Router) {
    super();
  }

  ngOnInit(): void {
    this.queryChanged
      .pipe(
        takeUntil(this.unsubscribe),
        debounceTime(500),
      )
      .subscribe(query => {
        this.query = query;
        this.applyFilter();
      });
  }

  onHideInvisibleChanged(val: boolean) {
    this.hideInvisible = val;
    this.hideInvisibleChanged.emit(val);
  }

  onQueryChange(query) {
    this.queryChanged.next(query);
  }

  applyFilter() {
    this.filteredExerciseProgresses = this.query 
      ? this.exerciseProgresses.filter(
          r => r.exercise.name.toLowerCase().indexOf(this.query.toLowerCase()) !== -1 ||
          r.taskInstances.some(ti => ti.task.name.toLocaleLowerCase().indexOf(this.query.toLowerCase()) !== -1)
      )
      : this.exerciseProgresses;
  }
}
