import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { BackendError } from 'app/shared/models/backend-error';
import { takeUntil } from 'rxjs/operators';
import { Exercise } from '../../models/exercise';
import { Submission } from '../../models/submission';
import { Origin } from '../../pages/submission/origin';
import { SubmissionService } from '../../services/submission.service';

@Component({
  selector: 'ngx-submission-selection',
  templateUrl: './submission-selection.component.html',
  styleUrls: ['./submission-selection.component.scss']
})
export class SubmissionSelectionComponent extends BaseComponent implements OnInit {
  @Input() exercise: Exercise;
  @Input() unitId: number;
  @Input() taskId: number;
  @Input() origin: Origin;
  @Input() selectedSubmission: Submission | null = null;
  @Output() submissionsLoaded: EventEmitter<Submission[]> = new EventEmitter<Submission[]>();

  submissions: Submission[] = [];
  loading: boolean = true;
  error: BackendError;

  constructor(
    private submissionService: SubmissionService) {
    super();
  }

  ngOnInit(): void {
    this.submissionService.getSubmissions({ exerciseId: this.exercise.id, unitId: this.unitId })
      .pipe(
        takeUntil(this.unsubscribe)
      ).subscribe(submissions => {
        this.submissions = submissions;
        this.loading = false;
        this.submissionsLoaded.emit(this.submissions);
      }, (error: any) => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }


  public update(submission: Submission) {
    let submissionInList = this.submissions.find(s => s.id == submission.id);
    if(submissionInList) {
      submissionInList.checkedBy = submission.checkedBy;
      submissionInList.passed = submission.passed;
    }
  }
}
