import { Exercise } from "app/features/exercises/models/exercise";
import { Submission } from "app/features/exercises/models/submission";
import { Solution } from "app/features/solutions/models/solution";
import { TaskInstance } from "./task-instance";

export class ExerciseProgress {
    exercise: Exercise;
    taskInstances: TaskInstance[];
    testsPassed: number;
    testsTotal: number;
    styleErrorsTotal: number;
    leaksTotal: number;
    submission: Submission;
    solutions: Solution[];

    constructor(exerciseProgress: any) {
        this.testsPassed = exerciseProgress.testsPassed;
        this.testsTotal = exerciseProgress.testsTotal;
        this.styleErrorsTotal = exerciseProgress.styleErrorsTotal;
        this.leaksTotal = exerciseProgress.leaksTotal;
        this.exercise = new Exercise(exerciseProgress.exercise);
        this.taskInstances = exerciseProgress.taskInstances.map(ti => new TaskInstance(ti));
        this.submission = exerciseProgress.submission ? new Submission(exerciseProgress.submission) : null;
        this.solutions = exerciseProgress.solutions.map(r => new Solution(r));
    }

    solutionCompiles(taskInstance: TaskInstance) {
        return this.hasSolution(taskInstance) && this.getSolution(taskInstance).hasResult && this.getSolution(taskInstance).evalResult.compiling
    }

    solutionDoesNotCompile(taskInstance: TaskInstance) {
        return this.hasSolution(taskInstance) && this.getSolution(taskInstance).hasResult && !this.getSolution(taskInstance).evalResult.compiling
    }

    solutionMissingOrNoResult(taskInstance: TaskInstance) {
        return !this.solutionPresentWithResult(taskInstance);
    }

    solutionPresentWithResult(taskInstance: TaskInstance) {
        return this.hasSolution(taskInstance) && this.getSolution(taskInstance).hasResult;
    }

    exercisePassed() {
        return this.exercise.usingBinary && this.submission && this.submission.passed == true;
    }

    exerciseNotPassed() {
        return this.exercise.usingBinary && this.submission && this.submission.passed == false;
    }

    exerciseNoDecision() {
        return this.exercise.usingBinary && this.submission && this.submission.passed == null;
    }

    hasSolution(taskInstance: TaskInstance) {
        return this.getSolution(taskInstance) != null;
    }

    getSolution(taskInstance: TaskInstance) {
        return this.solutions.find(s => s.task.id == taskInstance.task.id);
    }

    hasPointRating(taskInstance: TaskInstance) {
        return this.getPointRating(taskInstance) != null;
    }

    getPointRating(taskInstance: TaskInstance) {
        return this.submission?.pointRatings.find(r => r.taskInstance.task.id == taskInstance.task.id);
    }

}