
import { Course } from 'app/features/courses/models/course';
import { Iso8601Helper } from 'app/shared/helper/iso8601.helper';
import * as moment from 'moment';
import { AssessmentMode } from './assessment-mode';

export class Exercise {
    id: number;
    name: string;
    description: string;
    summary: string;
    assessmentMode: AssessmentMode;
    course: Course;
    dueTo: moment.Moment;
    visibleFrom: moment.Moment;
    reachablePoints: number;
    visible: boolean;
    taskInstancesCount: number;

    constructor(exercise: any) {
        this.id = exercise.id;
        this.name = exercise.name;
        this.description = exercise.description;
        this.summary = exercise.summary;
        this.assessmentMode = exercise.assessmentMode;
        this.reachablePoints = exercise.reachablePoints;
        this.course = exercise.course ? new Course(exercise.course) : null;
        this.dueTo = Iso8601Helper.fromString(exercise.dueTo);
        this.visibleFrom = Iso8601Helper.fromString( exercise.visibleFrom);
        this.visible = exercise.visible;
        this.taskInstancesCount = exercise.taskInstancesCount;
    }

    isActive(): boolean {
        return this.dueTo ? moment().isSameOrBefore(this.dueTo) : true;
    }

    get usingAssessmentMode(): boolean {
        return this.usingPoints || this.usingBinary;
    }

    get usingPoints(): boolean {
        return this.assessmentMode == AssessmentMode.Points;
    }

    get usingBinary(): boolean {
        return this.assessmentMode == AssessmentMode.Binary;
    }
}

