import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { AppStateService } from 'app/core/services/app.state.service';
import { AuthService } from 'app/core/services/auth.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { Breadcrumb } from 'app/shared/components/breadcrumbs/breadcrumbs.component';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { BackendError } from 'app/shared/models/backend-error';
import { combineLatest } from 'rxjs';
import { takeUntil, flatMap } from 'rxjs/operators';
import { TaskInstance } from '../../models/task-instance';
import { ExerciseService } from '../../services/exercise.service';
import { Origin } from '../submission/origin';

@Component({
  selector: 'ngx-task-instance-submissions',
  templateUrl: './task-instance-submissions.component.html',
  styleUrls: ['./task-instance-submissions.component.scss']
})
export class TaskInstanceSubmissionsComponent extends BaseComponent implements OnInit {
  loading: boolean;
  taskInstance: TaskInstance;
  
  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diese Übung konnten wir leider nicht finden.'
    }
  };

  breadcrumbs: Breadcrumb[] = [];
  Origin = Origin;

  constructor(
    private exerciseService: ExerciseService,
    private appStateService: AppStateService,
    private authService: AuthService,
    private route: ActivatedRoute,
    private router: Router) {
    super();
  }

  buildBreadcrumbs(taskInstance: TaskInstance) {
    this.breadcrumbs = [
      {
        id: 'exercises',
        params: {
          course_id: taskInstance.exercise.course.id
        },
      },
      {
        id: 'exercise',
        params: {
          exercise_id: taskInstance.exercise.id
        },
        ctx: taskInstance.exercise
      },
      {
        id: 'submitSolution',
        params: {
          exercise_id: taskInstance.exercise.id,
          task_id: taskInstance.task.id
        },
        ctx: taskInstance
      },
      {
        id: 'taskInstanceSubmissions',
        params: {
          exercise_id: taskInstance.exercise.id,
          task_id: taskInstance.task.id
        },
      },
      
    ];
  }
  
  ngOnInit(): void {
    this.loading = true;
    combineLatest(this.route.paramMap, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(([params, principal]) => {
          let exerciseId = Number(params.get("exercise_id"));
          let taskId = Number(params.get("task_id"));
          return this.exerciseService.getTaskInstance(exerciseId, taskId);
        }))
      .subscribe(taskInstance => {
        this.taskInstance = taskInstance;
        this.appStateService.setCourse(taskInstance.exercise.course);
        this.buildBreadcrumbs(taskInstance);

        this.loading = false;
      }, (error: any) => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }
}
