import { AfterViewInit, Component, Input, OnInit, QueryList, ViewChild, ViewChildren } from '@angular/core';
import { CodemirrorComponent } from '@ctrl/ngx-codemirror';
import { AppStateService } from 'app/core/services/app.state.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { takeUntil, take } from 'rxjs/operators';
import { Solution } from '../../models/solution';
import { EvalResult } from '../../models/eval-result';
import { CodemirrorHelper } from 'app/shared/helper/codemirror.helper';

@Component({
  selector: 'ngx-eval-result',
  templateUrl: './eval-result.component.html',
  styleUrls: ['./eval-result.component.scss']
})
export class EvalResultComponent extends BaseComponent implements OnInit {
  @ViewChildren(CodemirrorComponent) codemirrorComponents: QueryList<CodemirrorComponent>;
  @Input() evalResult: EvalResult;
  darkMode = false;

  constructor(
    private appStateService: AppStateService,
  ) { 
    super();
  }

  ngOnInit(): void {
    this.appStateService.darkMode$
    .pipe(takeUntil(this.unsubscribe))
    .subscribe(darkMode => this.darkMode = darkMode);
  }

  refreshEditors() {
    this.codemirrorComponents.forEach(component => {
      CodemirrorHelper.waitForEditor(component)
        .pipe(take(1))
        .subscribe(() => component.codeMirror.refresh());
    });
  }
}
