import { Component, Input, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { NbDialogService } from '@nebular/theme';
import { AuthService } from 'app/core/services/auth.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { BackendError } from 'app/shared/models/backend-error';
import { NotificationService } from 'app/shared/services/notification.service';
import { takeUntil } from 'rxjs/operators';
import { TaskFeedback } from '../../models/task-feedback';
import { TaskService } from '../../services/task.service';
import { Task } from 'app/features/tasks/models/task';

@Component({
  selector: 'ngx-task-feedback-info',
  templateUrl: './task-feedback-info.component.html',
  styleUrls: ['./task-feedback-info.component.scss']
})
export class TaskFeedbackInfoComponent extends BaseComponent implements OnInit {
  @Input() task: Task;

  feedback: TaskFeedback[] = [];
  loading: boolean = true;
  error: BackendError;

  constructor(
    private authService: AuthService,
    private taskService: TaskService,
    private dialogService: NbDialogService,
    private router: Router,
    private route: ActivatedRoute,
    private notificationService: NotificationService,
  ) {
    super();
  }

  ngOnInit(): void {
    this.loading = true;
    this.taskService.getFeedback(this.task.id)
      .pipe(
        takeUntil(this.unsubscribe),
      ).subscribe(feedback => {
          this.feedback = feedback;
          this.loading = false;
        }, error => {
          if (BackendError.isBackendError(error)) {
            this.error = error;
            this.loading = false;
          } else {
            throw error;
          }
      });
  }

  markAsRead(fb: TaskFeedback) {
    this.taskService.markFeedbackAsRead(this.task.id, fb.from.id)
    .pipe(
      takeUntil(this.unsubscribe)
    ).subscribe(() => {
      var idx = this.feedback.map(f => f.id).indexOf(fb.id);
      this.feedback.splice(idx, 1);
    });
  }
}
