import { Pipe, PipeTransform } from "@angular/core";


/**
 * If math expressions (Tex) were entered in the WYSIWYG editor, special characters (such as <) 
 * are escaped as html entities (&lt;) + html tags (like <br>) are inserted. We need to remove these
 * because the Katex renderer cannot parse this.   
 */
@Pipe({
    name: 'sanitize_tex',
})
export class SanitizeTexPipe implements PipeTransform {

    transform(inHtml: string): any {
        return this.removeHtmlInTex(inHtml);
    }

    // some teststrings:
    // \[x \le 17\]
    // \[\[x \le 17\]\]
    // $aaa$
    // $$aaa$$
    removeHtmlInTex(html: string): string {
        var newHtml = html;

        // Remove HTML tags and entities enclosed in $$...$$
        newHtml = newHtml.replace(/\${2}(.+?)\${2}/g, (match, p1) => {
            return '$$' + this.replaceHtmlEntities(this.replaceHtmlTags(p1)) + '$$';
        });

        // Remove HTML tags and entities enclosed in $...$ (does not match $$...$$)
        // wtf
        newHtml = newHtml.replace(/(?<!\$)\$(?!\$)(.*?)(?<!\$)\$(?!\$)/g, (match, p1) => {
            return '$' + this.replaceHtmlEntities(this.replaceHtmlTags(p1)) + '$';
        });

        // Remove HTML tags and entities enclosed in \[\[...\]\]
        // wtf wtf wtf
        newHtml = newHtml.replace(/(?:\\\[){2}(.+?)(?:\\\]){2}/g, (match, p1) => {
            return '\\[\\[' + this.replaceHtmlEntities(this.replaceHtmlTags(p1)) + '\\]\\]';
        });

        // Remove HTML tags and entities enclosed in \[...\] (does not match \[\[...\]\])
        // wtf^9000 - sorry 
        newHtml = newHtml.replace(/(?<!(?:\\\[))\\\[(?!(?:\\\[))(.*?)(?<!(?:\\\]))\\\](?!(?:\\\]))/g, (match, p1) => {
            return '\\[' + this.replaceHtmlEntities(this.replaceHtmlTags(p1)) + '\\]';
        });

        return newHtml;
    }

    private replaceHtmlTags(html: string) {
        return html.replace(/<\/?[^>]+(>|$)/g, '');
    }

    private replaceHtmlEntities(html: string) {
        const entities: { [key: string]: string } = {
            '&amp;': '&',
            '&lt;': '<',
            '&gt;': '>',
            '&quot;': '"',
            '&#x27;': "'",
            '&#x60;': '`'
        };

        return html.replace(/&(amp|lt|gt|quot|#x27|#x60);/g, (match, entity) => {
            return entities[match];
        });
    }

}