package de.hsrm.sls.subato.auth.principal;

import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional(readOnly = true)
public class PrincipalService implements UserDetailsService  {
    @Autowired
    UserService userService;
    @Autowired
    SubatoPrincipalMapper subatoPrincipalMapper;

    @Cacheable(value = "currentUser", key = "#username")
    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        var user = userService.findByLoginName(username);
        var subatoPrincipal = subatoPrincipalMapper.map(user);
        return subatoPrincipal;
    }

    @CacheEvict(value = "currentUser", key = "#user.loginName", condition="#user != null && #user.loginName != null")
    public void evictPrincipal(User user) {}

    @CacheEvict(value = "currentUser", key = "#principal.loginName", condition="#principal != null && #principal.loginName != null")
    public void evictPrincipal(SubatoPrincipal principal) {}

    public SubatoPrincipal getPrincipal(Authentication authentication) {
        return authentication != null && authentication.getPrincipal() instanceof SubatoPrincipal ? (SubatoPrincipal) authentication.getPrincipal() : null;
    }

    public SubatoPrincipal getPrincipal() {
        var principal = SecurityContextHolder.getContext().getAuthentication();
        return getPrincipal(principal);
    }
}
