package de.hsrm.sls.subato.exercises.submission.progress;

import de.hsrm.sls.subato.courses.course.Course;
import de.hsrm.sls.subato.courses.unit.CourseUnit;
import de.hsrm.sls.subato.courses.unit.CourseUnitService;
import de.hsrm.sls.subato.courses.unit.CourseUnitType;
import de.hsrm.sls.subato.exercises.exercise.Exercise;
import de.hsrm.sls.subato.exercises.submission.SubmissionFilter;
import de.hsrm.sls.subato.exercises.submission.SubmissionService;
import de.hsrm.sls.subato.user.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class SubmissionProgressService {
    @Autowired
    private SubmissionService submissionService;

    @Autowired
    private CourseUnitService courseUnitService;

    public SubmissionsProgress getProgressForUserInCourse(Course course, User user) {
        var filter = new SubmissionFilter().submittedById(user.getId()).courseId(course.getId());
        var submissions = submissionService.findAll(filter);
        return SubmissionsProgress.fromSubmissions(submissions);
    }

    public UnitSubmissionsProgress getProgressForExerciseAndUnit(Exercise exercise, CourseUnit unit) {
        var filter = new SubmissionFilter().exerciseId(exercise.getId()).unitId(unit.getId());
        var submissions = submissionService.findAll(filter);
        submissions.sort(Comparator.comparing(submission -> submission.getSubmittedBy().getName()));

        var unitProgress = new UnitSubmissionsProgress();
        unitProgress.setUnit(unit);
        unitProgress.setProgress(SubmissionsProgress.fromSubmissions(submissions));

        var nextSubmission = submissions.stream().filter(s -> s.getPassed() == null).findFirst().orElse(
                submissions.stream().findFirst().orElse(null)
        );

        if(nextSubmission != null) {
            unitProgress.setNextSubmissionId(nextSubmission.getId());
        }

        return unitProgress;
    }

    public List<UnitSubmissionsProgress> getProgressForExercise(Exercise exercise) {
        var units = courseUnitService.findByCourse(exercise.getCourse()).stream()
                .filter(u -> u.getType() == CourseUnitType.Lab || u.getType() == CourseUnitType.Misc)
                .collect(Collectors.toList());

        var unitProgresses = new ArrayList<UnitSubmissionsProgress>();

        for(var unit : units) {
            var unitProgress = getProgressForExerciseAndUnit(exercise, unit);
            unitProgresses.add(unitProgress);
        }

        return unitProgresses;
    }
}
