package de.hsrm.sls.subato.tasks.lang;

import de.hsrm.sls.subato.shared.ExceptionReasons;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;
import java.util.stream.Collectors;

import static org.springframework.http.HttpStatus.NOT_FOUND;

@RestController
@Tag(name = "Aufgabenverwaltung")
public class LanguageController {
    @Autowired
    private LanguageService languageService;

    @Autowired
    private LanguageMapper languageMapper;

    @GetMapping("/language/{id}")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "404", description = LanguageNotFoundException.REASON),
    })
    LanguageDto getOne(@PathVariable String id) {
        try {
            var language = languageService.findById(id);
            return languageMapper.map(language);
        } catch(LanguageNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, LanguageNotFoundException.REASON);
        }
    }

    @GetMapping("/language")
    List<LanguageDto> getAll() {
        var languages = languageService.findAll();
        return languages.stream().map(lang -> languageMapper.mapWithCount(lang)).collect(Collectors.toList());
    }

    @PostMapping("/language")
    @PreAuthorize("hasPermission(null, @TasksAction.LANGUAGE_CREATE)")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "400", description = ExceptionReasons.CONSTRAINT_VALIDATION),
    })
    LanguageDto create(@RequestBody CreateLanguageDto dto) {
        var language = languageService.create(dto);
        return languageMapper.map(language);
    }

    @PutMapping ("/language/{id}")
    @PreAuthorize("hasPermission(null, @TasksAction.LANGUAGE_UPDATE)")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "400", description = ExceptionReasons.CONSTRAINT_VALIDATION),
            @ApiResponse(responseCode = "404", description = LanguageNotFoundException.REASON),
    })
    LanguageDto update(@PathVariable String id, @RequestBody CreateLanguageDto dto) {
        try {
            var language = languageService.update(id, dto);
            return languageMapper.map(language);
        } catch(LanguageNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, LanguageNotFoundException.REASON);
        }
    }

    @DeleteMapping("/language/{id}")
    @PreAuthorize("hasPermission(null, @TasksAction.LANGUAGE_DELETE)")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "404", description = LanguageNotFoundException.REASON),
    })
    void delete(@PathVariable String id) {
        try {
            languageService.delete(id);
        } catch(LanguageNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, LanguageNotFoundException.REASON);
        }
    }
}
