package de.hsrm.sls.subato.tasks.stef.cache;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.ObjectCodec;
import com.fasterxml.jackson.databind.*;
import de.hsrm.sls.subato.tasks.stef.spec.StefSpec;

import java.io.IOException;

/**
 * Checks if the serialized {@link StefSpec} can be parsed by inspecting the {@link StefSpecCache#serializationVersion}.
 * If it does not match with {@link StefSpecCache#SERIALIZATION_VERSION}, the spec won't be read and should be rebuilt
 */
public class StefSpecCacheDeserializer extends JsonDeserializer<StefSpecCache> {
    @Override
    public StefSpecCache deserialize(JsonParser p, DeserializationContext ctxt) throws IOException {
        ObjectCodec codec = p.getCodec();
        JsonNode node = codec.readTree(p);

        if(!node.has("serializationVersion")) {
            return null;
        }

        int version = node.get("serializationVersion").asInt();
        if (version == StefSpecCache.SERIALIZATION_VERSION) {
            var spec = codec.treeToValue(node.get("spec"), StefSpec.class);
            return new StefSpecCache(version, spec);
        } else {
            return null;
        }
    }
}

