package de.hsrm.sls.subato.tasks.stef.parser.meta;

import de.hsrm.sls.subato.tasks.stef.spec.Meta;
import de.hsrm.sls.subato.tasks.stef.parser.InvalidSpecException;
import de.hsrm.sls.subato.tasks.stef.parser.meta.xml.TaskXml;
import jakarta.xml.bind.JAXBContext;
import jakarta.xml.bind.JAXBException;
import jakarta.xml.bind.Unmarshaller;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


import java.nio.file.Path;

@Service
public class MetaParser {
    @Autowired
    private IdExtractor idExtractor;

    @Autowired
    private NameExtractor nameExtractor;

    @Autowired
    private DescriptionExtractor descriptionExtractor;

    @Autowired
    private AttemptsExtractor attemptsExtractor;

    @Autowired
    private LanguageExtractor languageExtractor;

    @Autowired
    private LegacyIdExtractor legacyIdExtractor;

    @Autowired
    private SubmissionModeExtractor submissionModeExtractor;

    @Autowired
    private EvaluatorExtractor evaluatorExtractor;

    @Autowired
    private FileRefsExtractor fileRefsExtractor;

    @Autowired
    private MimeMappingsExtractor mimeMappingsExtractor;

    public Meta parse(String fileRoot) {
        var meta = new Meta();

        var doc = parseXmlDocument(fileRoot);

        idExtractor.extract(doc, meta);
        nameExtractor.extract(doc, meta);
        languageExtractor.extract(doc, meta);
        descriptionExtractor.extract(doc, meta);
        attemptsExtractor.extract(doc, meta);
        legacyIdExtractor.extract(doc, meta);
        evaluatorExtractor.extract(doc, meta);
        mimeMappingsExtractor.extract(doc, meta);
        fileRefsExtractor.extract(doc, meta, fileRoot);
        submissionModeExtractor.extract(doc, meta);

        return meta;
    }

    private TaskXml parseXmlDocument(String fileRoot)  {
        var metaFile = Path.of(fileRoot).resolve("meta.xml").toFile();
        if(!metaFile.exists()) {
            throw new InvalidSpecException("Die Datei meta.xml existiert nicht im Wurzelverzeichnis, ist jedoch erforderlich");
        }

        try {
            JAXBContext jaxbContext = JAXBContext.newInstance(TaskXml.class);
            Unmarshaller jaxbUnmarshaller = jaxbContext.createUnmarshaller();
            return (TaskXml) jaxbUnmarshaller.unmarshal(metaFile);
        } catch(JAXBException e) {
            throw new InvalidSpecException("Die Datei meta.xml ist syntaktisch ungültig: %s".formatted(e.getMessage()));
        }
    }
}
