package de.hsrm.sls.subato.tasks.task;

import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import jakarta.servlet.http.HttpServletRequest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.net.URI;
import java.net.URISyntaxException;
import java.nio.file.Path;
import java.util.regex.Pattern;

/**
 * Fix relative urls to assets in the html content of the task,
 * so that the url points to our endpoint, serving the asset
 * @see TaskController#downloadAsset
 */
@Service
public class TaskReferenceRewriter {
    private static Logger logger = LoggerFactory.getLogger(TaskReferenceRewriter.class);

    String getBaseUrl(HttpServletRequest request) {
        return ""
                + request.getScheme() + "://"
                + request.getServerName()
                + ":" + request.getServerPort();
    }

    private String rewriteUrls(String content, String path, HttpServletRequest request) {
        var url = "%s/%s".formatted(getBaseUrl(request), path);

        var matcher = Pattern.compile("src=\"(\\S+)\"").matcher(content);
        return matcher.replaceAll(r -> {
            var matchedUrl = r.group(1);
            var correctedUrl = matchedUrl;

            try {
                var uri = new URI(matchedUrl);
                if(!uri.isAbsolute()) {
                    correctedUrl ="%s%s".formatted(url, matchedUrl);
                }
            } catch (URISyntaxException e) {
                // there is an incorrect url in the task text, thus this is no error
            }

            return "src=\"%s\"".formatted(correctedUrl);
        });
    }

    public String rewrite(Task task, HttpServletRequest request) {
        if(task.isArchived()) {
            throw new TaskArchivedException();
        }

        var taskAssetPath = Path.of(
                "task",
                String.valueOf(task.getId()),
                "asset?path=").toString();
        var spec = task.getSpec();
        return rewriteUrls(spec.getText(), taskAssetPath, request);
    }

    public String rewrite(TaskInstance taskInstance, HttpServletRequest request) {
        if(taskInstance.getTask() != null) {
            if(taskInstance.getTask().isArchived()) {
                throw new TaskArchivedException();
            }

            var taskAssetPath = Path.of(
                    "exercise",
                    String.valueOf(taskInstance.getExercise().getId()),
                    "task",
                    String.valueOf(taskInstance.getTask().getId()), "asset?path=").toString();

            var spec = taskInstance.getTask().getSpec();
            return rewriteUrls(spec.getText(), taskAssetPath, request);
        }

        return "";
    }
}