package de.hsrm.sls.subato.user;


import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.users.UsersAction;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.web.PageableDefault;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.server.ResponseStatusException;

import java.util.Optional;

import static org.springframework.http.HttpStatus.NOT_FOUND;

@RestController
@Tag(name = "Querschnitt")
public class UserController {
    @Autowired
    private UserService userService;

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private PolicyEvaluator policy;

    @GetMapping("/user/{id}")
    @Operation(summary = "Liefert Informationen zum angegebenen Benutzer")
    UserDetailDto getOne(@PathVariable Long id) {
        try {
            var user = userService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(user, UsersAction.USER_VIEW));

            return userMapper.mapDetail(user);
        } catch(UserNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, "Der Benutzer existiert nicht");
        }
    }

    @GetMapping("/user")
    Page<UserDetailDto> findAll(
            @PageableDefault Pageable page,
            @RequestParam("q") Optional<String> query,
            @RequestParam Optional<Boolean> active
        ) {
        var filter = new UserFilter()
                .query(query.orElse(null))
                .active(active.orElse(null));
        var updatedPage = PageRequest.of(page.getPageNumber(), page.getPageSize(), userService.nameSort());

        var users = userService.findAll(updatedPage, filter, UsersAction.USER_VIEW);
        return users.map(u -> userMapper.mapDetail(u));
    }
}
